import 'package:flutter/material.dart';
import 'package:hub/backend/schema/enums/enums.dart';
import 'package:hub/components/atomic_components/menu_button_item/menu_button_item_widget.dart';
import 'package:hub/components/atomic_components/menu_card_item/menu_card_item.dart';
import 'package:hub/components/molecular_components/menu_item/menu_item.dart';
import 'package:hub/components/molecular_components/option_selection_modal/option_selection_modal_widget.dart';
import 'package:hub/components/organism_components/menu_component/menu_component_widget.dart';
import 'package:hub/components/organism_components/menu_list_view_component/menu_list_view_component_model.dart';
import 'package:hub/components/organism_components/menu_staggered_view_component/menu_staggered_view_component_model.dart';
import 'package:hub/flutter_flow/flutter_flow_util.dart';
import 'package:hub/flutter_flow/nav/nav.dart';
import 'package:hub/shared/extensions/dialog_extensions.dart';
import 'package:hub/shared/helpers/base_storage.dart';
import 'package:hub/shared/helpers/storage_helper.dart';
import 'package:hub/shared/services/authentication/authentication_service.dart';
import 'package:material_symbols_icons/symbols.dart';

import '/components/organism_components/menu_list_view_component/menu_list_view_component_widget.dart';
import 'menu_component_widget.dart' show MenuComponentWidget;

class MenuComponentModel extends FlutterFlowModel<MenuComponentWidget> {
  final MenuView style;
  final MenuItem item;
  final bool expandable;
  final List<MenuOption> menuOptions;

  MenuComponentModel({required this.style, required this.item, required this.expandable, required this.menuOptions});

  bool isGrid = false;

  late MenuListViewComponentModel menuListViewComponentModel;
  late MenuStaggeredViewComponentModel menuStaggeredViewComponentModel;
  late VoidCallback safeSetState;
  late List<MenuEntry?> menuEntries;

  @override
  void initState(BuildContext context) {
    menuListViewComponentModel = createModel(context, () => MenuListViewComponentModel());
    menuStaggeredViewComponentModel = createModel(context, () => MenuStaggeredViewComponentModel());
  }

  @override
  void dispose() {
    menuListViewComponentModel.dispose();
    menuStaggeredViewComponentModel.dispose();
  }

  List<MenuEntry?> generateMenuEntries(BuildContext context, MenuItem item, List<MenuOption> options) {
    List<MenuEntry?> entries = [];
    MenuEntry? addMenuEntry(IconData icon, String enText, String ptText, Function() action) {
      entries.add(
        item == MenuItem.button
            ? MenuButtonWidget(
                icon: icon,
                action: action,
                title: FFLocalizations.of(context).getVariableText(enText: enText, ptText: ptText),
                safeSetState: safeSetState)
            : item == MenuItem.card
                ? MenuCardItem(
                    icon: icon,
                    action: action,
                    title: FFLocalizations.of(context).getVariableText(enText: enText, ptText: ptText),
                    safeSetState: safeSetState)
                : item == MenuItem.tile
                    ? MenuCardItem(
                        icon: icon,
                        action: action,
                        title: FFLocalizations.of(context).getVariableText(enText: enText, ptText: ptText),
                        safeSetState: safeSetState)
                    : null,
      );
      return null;
    }

    /// ABOUT THE VISITORS
    if (options.contains(MenuOption.VisitorsRegister))
      addMenuEntry(Icons.person_add_alt_1_outlined, 'Register Visitor', 'Cadastrar Visitante', () async {
        await open(context, '/registerVisitorPage');
        safeSetState;
      });
    if (options.contains(MenuOption.WorkersOnTheProperty))
      addMenuEntry(Icons.engineering_outlined, 'Schedule Providers', 'Agendar Prestadores', () async {
        if (await StorageHelper().get(SQLiteStorageKey.provisional.value, Storage.SQLiteStorage) == 'true')
          await open(context, '/provisionalSchedule');
        else
          DialogUnavailable.unavailableFeature(context);
        safeSetState;
      });
    if (options.contains(MenuOption.DeliverySchedule))
      addMenuEntry(Icons.sports_motorsports_outlined, 'Schedule Deliveries', 'Agendar Entregas', () async {
        if (await StorageHelper().get(SQLiteStorageKey.provisional.value, Storage.SQLiteStorage) == 'true')
          await open(context, '/deliverySchedule');
        else
          DialogUnavailable.unavailableFeature(context);
        safeSetState;
      });
    if (options.contains(MenuOption.FastPassSchedule))
      addMenuEntry(Icons.attach_email_outlined, 'Invite Visitor', 'Convidar Visitante', () async {
        if (await StorageHelper().get(SQLiteStorageKey.whatsapp.value, Storage.SQLiteStorage) == 'true')
          await open(context, '/fastPassPage');
        else
          DialogUnavailable.unavailableFeature(context);
        safeSetState;
      });
    if (options.contains(MenuOption.CompleteSchedule))
      addMenuEntry(Icons.event, 'Complete Schedule', 'Agenda Completa', () async {
        await open(context, '/scheduleCompleteVisitPage');
        safeSetState;
      });

    /// ABOUT THE PROPERTY
    if (options.contains(MenuOption.PetsOnTheProperty))
      addMenuEntry(Icons.pets, 'Pets', 'Pets', () async {
        if (await StorageHelper().get(SQLiteStorageKey.pets.value, Storage.SQLiteStorage) == 'true')
          await open(context, '/petsOnThePropertyPage');
        else
          DialogUnavailable.unavailableFeature(context);
        safeSetState;
      });
    if (options.contains(MenuOption.ResidentsOnTheProperty))
      addMenuEntry(Icons.groups, 'Residents', 'Moradores', () async {
        await open(context, '/peopleOnThePropertyPage');
        safeSetState;
      });
    if (options.contains(MenuOption.VehiclesOnTheProperty))
      addMenuEntry(Symbols.directions_car, 'Vehicles', 'Veículos', () async {
        await open(context, '/vehiclesOnThePropertyPage');
        safeSetState;
      });
    if (options.contains(MenuOption.VisitsOnTheProperty))
      addMenuEntry(Symbols.perm_contact_calendar, 'Opened Visits', 'Visitas em Aberto', () async {
        await open(context, '/visitsOnThePropertyPage');
        safeSetState;
      });
    if (options.contains(MenuOption.PackagesOnTheProperty))
      addMenuEntry(Icons.inventory_2_outlined, 'My Orders', 'Minhas Encomendas', () async {
        if (await StorageHelper().get(SQLiteStorageKey.whatsapp.value, Storage.SQLiteStorage) == 'true')
          await open(context, '/packageOrder');
        else
          DialogUnavailable.unavailableFeature(context);
        safeSetState;
      });

    /// ABOUT THE RESIDENTS
    if (options.contains(MenuOption.PetsRegister))
      addMenuEntry(Icons.pets, 'Pets Register', 'Cadastrar Pet', () async {
        if (await StorageHelper().get(SQLiteStorageKey.pets.value, Storage.SQLiteStorage) == 'true')
          await open(context, '/petsPage');
        else
          DialogUnavailable.unavailableFeature(context);
        safeSetState;
      });
    if (options.contains(MenuOption.ReservationsOnTheLocal))
      addMenuEntry(Icons.event_available, 'Reservations', 'Reservas', () async {
        if (await StorageHelper().get(SQLiteStorageKey.whatsapp.value, Storage.SQLiteStorage) == 'true')
          await open(context, '/reservation');
        else
          DialogUnavailable.unavailableFeature(context);

        safeSetState;
      });
    if (options.contains(MenuOption.QRCodeAccessInProperty))
      addMenuEntry(Icons.qr_code, 'QRCode Access', 'QRCode de Acesso', () async {
        await open(context, '/qrCodePage');
        safeSetState;
      });
    if (options.contains(MenuOption.AccessOnTheProperty))
      addMenuEntry(Icons.transfer_within_a_station_outlined, 'Access History', 'Consultar Acessos', () async {
        await open(context, '/acessHistoryPage');
        safeSetState;
      });
    if (options.contains(MenuOption.LiberationsOnTheProperty))
      addMenuEntry(Icons.how_to_reg_outlined, 'Liberations History', 'Consultar Liberações', () async {
        await open(context, '/liberationHistory');
        safeSetState;
      });
    if (options.contains(MenuOption.MessagesOnTheProperty))
      addMenuEntry(Icons.chat_outlined, 'Message History', 'Consultar Mensagens', () async {
        await open(context, '/messageHistoryPage');
        safeSetState;
      });

    /// ABOUT THE SYSTEM
    if (options.contains(MenuOption.AboutProperty))
      addMenuEntry(Icons.home, 'About Property', 'Sobre a Propriedade', () async {
        if (await StorageHelper().get(SQLiteStorageKey.whatsapp.value, Storage.SQLiteStorage) == 'true')
          await open(context, '/aboutProperty');
        else
          DialogUnavailable.unavailableFeature(context);
        safeSetState;
      });
    if (options.contains(MenuOption.SettingsOnTheApp))
      addMenuEntry(Icons.settings, 'System Settings', 'Opções do Sistema', () async {
        await open(context, '/preferencesSettings');
        safeSetState;
      });
    if (options.contains(MenuOption.LogoutOnTheApp))
      addMenuEntry(Icons.logout, 'Logout', 'Sair', () async {
        await out(context);
        safeSetState;
      });

    return entries;
  }

  Future open(BuildContext context, String link) async {
    context.push(
      link,
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

   Future out(BuildContext context) async {
    final String title = FFLocalizations.of(context).getVariableText(
      ptText: 'Sair',
      enText: 'Logout',
    );
    final String content = FFLocalizations.of(context).getVariableText(
      ptText: 'Tem certeza que deseja sair?',
      enText: 'Are you sure you want to logout?',
    );
    showAlertDialog(context, title, content, () async => await AuthenticationService.signOut(context));
  }

  Future openDeliverySchedule(BuildContext context) async {
    final bool isProvisional =
        await StorageHelper().get(SQLiteStorageKey.provisional.value, Storage.SQLiteStorage) == 'true';
    if (isProvisional == true) {
      context.push(
        '/deliverySchedule',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }

  Future openProvisionalSchedule(BuildContext context) async {
    final isProvisional =
        await StorageHelper().get(SQLiteStorageKey.provisional.value, Storage.SQLiteStorage) == 'true';
    if (isProvisional == true) {
      context.push(
        '/provisionalSchedule',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }

  Future openFastPassSchedule(BuildContext context) async {
    final isWpp = await StorageHelper().get(SQLiteStorageKey.whatsapp.value, Storage.SQLiteStorage) == 'true';
    if (isWpp) {
      context.push(
        '/fastPassPage',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }

  Future showSchedules(BuildContext context) async {
    final routesListStr = <String>[
      'scheduleProvisionalVisitPage',
      'fastPassPage',
      'scheduleCompleteVisitPage',
    ];
    final iconsListIcon = <IconData>[
      Icons.date_range_rounded,
      Icons.date_range_rounded,
      Icons.date_range_rounded,
    ];
    final nameListStr = <String>[
      FFLocalizations.of(context).getVariableText(
        ptText: 'Visita\nProvisória',
        enText: 'Provisional\nSchedule',
      ),
      FFLocalizations.of(context).getVariableText(
        ptText: 'Visita\nRápida',
        enText: 'Fast\nSchedule',
      ),
      FFLocalizations.of(context).getVariableText(
        ptText: 'Visita\nCompleta',
        enText: 'Complete\nSchedule',
      ),
    ];
    await showAdaptiveDialog(
      context: context,
      builder: (context) {
        return Padding(
          padding: MediaQuery.viewInsetsOf(context),
          child: OptionSelectionModalWidget(
            routesListStr: routesListStr,
            iconsListIcon: iconsListIcon,
            nameListStr: nameListStr,
          ),
        );
      },
    );
  }

  Future openVisitorsRegister(BuildContext context) async {
    context.push(
      '/registerVisitorPage',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openPoepleOnTheProperty(BuildContext context) async {
    context.push(
      '/peopleOnThePropertyPage',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future<void> Logout(BuildContext context) async {
    final String title = FFLocalizations.of(context).getVariableText(
      ptText: 'Sair',
      enText: 'Logout',
    );
    final String content = FFLocalizations.of(context).getVariableText(
      ptText: 'Tem certeza que deseja sair?',
      enText: 'Are you sure you want to logout?',
    );
    showAlertDialog(context, title, content, () async => await AuthenticationService.signOut(context));
  }

  Future openPreferencesSettings(BuildContext context) async {
    context.push(
      '/preferencesSettings',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openMyOrders(BuildContext context) async {
    final isWpp = await StorageHelper().get(SQLiteStorageKey.whatsapp.value, Storage.SQLiteStorage) == 'true';
    ;
    if (isWpp) {
      context.push(
        '/packageOrder',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }

  Future openReservations(BuildContext context) async {
    final isWpp = await StorageHelper().get(SQLiteStorageKey.whatsapp.value, Storage.SQLiteStorage) == 'true';
    ;
    if (isWpp) {
      context.push(
        '/reservation',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }

  Future showHistories(BuildContext context) async {
    await showAdaptiveDialog(
      // isScrollControlled: true,
      // backgroundColor: Colors.transparent,
      // enableDrag: false,
      context: context,
      builder: (context) {
        return Padding(
          padding: MediaQuery.viewInsetsOf(context),
          child: OptionSelectionModalWidget(
            routesListStr: const <String>[
              'liberationHistory',
              'acessHistoryPage',
              'scheduleCompleteVisitPage',
              'messageHistoryPage'
            ],
            iconsListIcon: const <IconData>[
              Icons.history_rounded,
              Icons.history_rounded,
              Icons.history_rounded,
              Icons.history_rounded,
            ],
            nameListStr: <String>[
              FFLocalizations.of(context).getVariableText(
                ptText: 'Histórico\nde Liberação',
                enText: 'Liberation\nHistory',
              ),
              FFLocalizations.of(context).getVariableText(
                ptText: 'Histórico\nde Acesso',
                enText: 'Access\nHistory',
              ),
              FFLocalizations.of(context).getVariableText(
                ptText: 'Histórico\nde Visita',
                enText: 'Visit\nHistory',
              ),
              FFLocalizations.of(context).getVariableText(
                ptText: 'Histórico\nde Mensagens',
                enText: 'Message\nHistory',
              ),
            ],
          ),
        );
      },
    );
  }

  Future openLiberationsHistory(BuildContext context) async {
    context.push(
      '/liberationHistory',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openAccessHistory(BuildContext context) async {
    context.push(
      '/acessHistoryPage',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openVisitsHistory(BuildContext context) async {
    context.push(
      '/scheduleCompleteVisitPage',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openMessagesHistory(BuildContext context) async {
    context.push(
      '/messageHistoryPage',
      extra: <String, dynamic>{
        kTransitionInfoKey: const TransitionInfo(
          hasTransition: false,
          transitionType: PageTransitionType.scale,
          alignment: Alignment.bottomCenter,
        ),
      },
    );
  }

  Future openPetsRegister(BuildContext context) async {
    bool isPet = await StorageHelper().get(SQLiteStorageKey.pets.value, Storage.SQLiteStorage) == 'true';
    ;
    if (isPet) {
      context.push(
        '/petsPage',
        extra: <String, dynamic>{
          kTransitionInfoKey: const TransitionInfo(
            hasTransition: false,
            transitionType: PageTransitionType.scale,
            alignment: Alignment.bottomCenter,
          ),
        },
      );
    } else {
      DialogUnavailable.unavailableFeature(context);
    }
  }
}
